//=======================================================================
// congruences.cc : recherche exhaustive de congruences modulaires
// Copyright (C) 2006-2007 by Michael Eisermann, all rights reserved
// Institut Fourier, Universit Joseph Fourier, Grenoble, France
//
// Description :
// Ce petit programme cherche les plus petites solutions du systme
//   a * b * c == 1  mod  d
//   a * b * d == 1  mod  c
//   a * c * d == 1  mod  b
//   b * c * d == 1  mod  a
// en parcourant btement quatre boucles imbriques.
//
// Remarque sur la compilation :
// Compiler avec  g++ congruences.cc -O3 -static  pour optimiser :
// - L'option -O3 fait que le compilateur applique quelques optimisations.
//   L'excution sera plus rapide, mais la compilation est plus complexe,
//   elle durera donc plus longtemps (de manire insignifiante ici).
//   Le logiciel ainsi compil sera un peu plus grand en gnral.
// - L'option -static fait inclure les bibliothques d'une manire dite
//   "statique". Ceci augmente considrablement la taille du logiciel,
//   mais l'acclre un peu.
// Pour voir les diffrences, compiler avec les 4 variantes possibles,
// puis comparer la taille et la rapidit du logiciel qui en rsulte.
// Pour en savoir plus, consultez le manuel en ligne de g++ en tapant
// "info gcc Invoking Optimize"  dans une fentre xterm.
//=======================================================================

#include <iostream>     // dclarer l'entre-sortie standard
using namespace std;    // accs direct aux fonctions standard

typedef long long int Integer;

int main()
{
  for( Integer d=5; d<1000; ++d )
    {
      cout << "d=" << d << "\r" << flush;
      for( Integer c=4; c<d; ++c )
	{
	  for( Integer b=3; b<c; ++b )
	    {
	      for( Integer a=2; a<b; ++a )
		{
		  if ( (a*b*c)%d == 1 && (a*b*d)%c == 1 && (a*c*d)%b == 1 && (b*c*d)%a == 1 )
		    cout << "solution (" << a << "," << b << "," << c << "," << d << ")" << endl;
		}
	    }
	}
    }
  cout << "Au revoir.   " << endl;
}
