//=======================================================================
// precision.cc : dterminer la prcision de float, double, long double
// Copyright (C) 2003-2007 Michael Eisermann, all rights reserved
// Institut Fourier, Universit Joseph Fourier, Grenoble, France
//=======================================================================

#include <iostream>   // dclarer l'entre-sortie standard
#include <iomanip>    // manipulateurs pour formater les flots
using namespace std;  // accs direct aux fonctions standard

// On a le choix entre une information concise ou verbeuse
template <typename flottant> void info_concise();
template <typename flottant> void info_verbeuse();
template <typename flottant> void info() 
{ info_concise<flottant>(); }

int main()
{
  cout << "\nTest empirique du type float :" << endl;
  info<float>();
  
  cout << "\nTest empirique du type double :" << endl;
  info<double>();

  cout << "\nTest empirique du type long double :" << endl;
  info<long double>();
}

template <typename flottant>
void info_concise()
{
  // Afficher le nombre d'octets occup dans la mmoire
  cout << "Une variable de ce type occupe " 
       << sizeof(flottant) << " octets = "
       << 8*sizeof(flottant) << " bits." << endl;

  // Dterminer la longueur de la mantisse.  Pour ceci
  // on dtermine eps = 2^(-n) tel que 1.0 + eps = 1.0
  cout << "Mantisse ";
  flottant eps= 1.0;
  int mbits;
  for ( mbits= 1; mbits<1000; ++mbits )
    {
      cout << ".";
      eps/= 2;
      flottant somme= 1.0 + eps;
      if ( somme == 1.0 ) break;
    };
  cout << " " << mbits << " bits" << endl;

  // Dterminer l'exposant maximal
  cout << "Exposant .";
  flottant puissance= 2.0;
  int ebits=1, exposant=1;
  while ( 2*puissance != puissance )
    {
      cout << ".";
      ++ebits;
      puissance*= puissance * 2.0;
      exposant= 2*exposant+1;
    };
  cout << " " << ebits << " bits" << endl;
}

template <typename flottant>
void info_verbeuse()
{
  // Afficher le nombre d'octets occup dans la mmoire
  cout << "Une variable de ce type occupe " 
       << sizeof(flottant) << " octets = "
       << 8*sizeof(flottant) << " bits.\n\n"
       << "Tester la prcision de la mantisse : " << endl;

  // Dterminer la longueur de la mantisse.  Pour ceci
  // on dtermine eps = 2^(-n) tel que 1.0 + eps = 1.0
  flottant eps= 1.0;
  int mbits;
  for ( mbits= 1; mbits<1000; ++mbits )
    {
      eps/= 2;
      flottant somme= 1.0 + eps;
      if ( somme > 1.0 )
	{
	  cout << "\r" << setw(3) << mbits+1 << " bits ? 1+eps > 1  ok     "
	       << "avec eps = 2^(-" << mbits << ") = " << eps << flush;
	}
      else
	{
	  cout << "\n" << setw(3) << mbits+1 << " bits ? 1+eps == 1 erreur "
	       << "avec eps = 2^(-" << mbits << ") = " << eps << flush;
	  break;
	};
    };
  cout << "\nOn a donc " << mbits << " bits pour la mantisse." << endl;

  // Dterminer l'exposant maximal
  cout << endl << "Tester la capacit de l'exposant : " << endl;
  flottant puissance= 2.0;
  int ebits=1, exposant=1;
  while ( 2*puissance != puissance )
    {
      puissance*= puissance * 2.0;
      exposant= 2*exposant+1;
      cout << setw(3) << ++ebits << " bits ?  2^" << exposant 
	   << " == " << puissance << endl;
    };
  cout << "On a donc 1+" << ebits-1 << " bits pour l'exposant (avec signe)." << endl;

  // Dterminer l'exposant minimal
  cout << endl << "Tester la capacit de l'exposant : " << endl;
  puissance= 0.5;
  ebits=1, exposant=-1;
  while ( 2*puissance != puissance )
    {
      puissance*= puissance;
      exposant= 2*exposant;
      cout << setw(3) << ++ebits << " bits ?  2^" << exposant 
	   << " == " << puissance << endl;
    };
  cout << "On a donc " << ebits-1 << " bits pour l'exposant (avec signe).\n" << endl;
}
