//=======================================================================
// naturel-test.cc : tester la correction et la performance
// Copyright (C) 2004-2007 by Michael Eisermann, all rights reserved
// Institut Fourier, Universit Joseph Fourier, Grenoble, France
//
// Compilation :
// Ajouter l'option -O3 pour une compilation optimisant la rapidit. 
//=======================================================================

#include <iostream>         // dclarer l'entre-sortie standard
#include <iomanip>          // manipulateurs pour formater les flots
#include "naturel.cc"       // dfinir le type Naturel, fait maison
#include "timer.hh"         // horloge pour chronomtrer l'excution
using namespace std;        // accs direct aux fonctions standard

void test_algorithmes_scolaires( const Naturel& a, const Naturel& b )
{
  cout << "\nTest des algorithmes scolaires :" << endl;
  cout << "a = " << a << endl;
  cout << "b = " << b << endl;
  
  cout << "Addition : " << flush;
  {
    Naturel somme;
    Timer timer;
    bool ok= addition_scolaire(a,b,somme);
    timer.stop();
    if ( !ok ) cout << "echec";
    else cout << "a+b = " << somme; 
    cout << ", temps coul : " << timer << endl;
  };

  cout << "Soustraction : " << flush;
  {
    Naturel diff;
    Timer timer;
    bool ok= soustraction_scolaire(a,b,diff);
    timer.stop();
    if ( !ok ) cout << "echec";
    else cout << "a-b = " << diff; 
    cout << ", temps coul : " << timer << endl;
  };
  
  cout << "Multiplication : " << flush;
  {
    Naturel produit;
    Timer timer;
    bool ok= multiplication_scolaire(a,b,produit);
    timer.stop();
    if ( !ok ) cout << "echec";
    else cout << "a*b = " << produit; 
    cout << ", temps coul : " << timer << endl;
  };

  cout << "Division euclidienne : " << flush;
  {
    Naturel quot, reste;
    Timer timer;
    bool ok= division_scolaire(a,b,quot,reste);
    timer.stop();
    if ( !ok ) cout << "echec";
    else cout << "a/b = " << quot << ", a%b = " << reste; 
    cout << ", temps coul : " << timer << endl;
  };
}

void test_algorithmes_lents( const Naturel& a, const Naturel& b )
{
  cout << "\nTest des algorithmes lents :" << endl;
  cout << "a = " << a << endl;
  cout << "b = " << b << endl;
  
  cout << "Addition : " << flush;
  {
    Naturel somme;
    Timer timer;
    bool ok= addition_lente(a,b,somme);
    timer.stop();
    if ( !ok ) cout << "echec";
    else cout << "a+b = " << somme; 
    cout << ", temps coul : " << timer << endl;
  };

  cout << "Soustraction : " << flush;
  {
    Naturel diff;
    Timer timer;
    bool ok= soustraction_lente(a,b,diff);
    timer.stop();
    if ( !ok ) cout << "echec";
    else cout << "a-b = " << diff; 
    cout << ", temps coul : " << timer << endl;
  };
  
  cout << "Multiplication : " << flush;
  {
    Naturel produit;
    Timer timer;
    bool ok= multiplication_lente(a,b,produit);
    timer.stop();
    if ( !ok ) cout << "echec";
    else cout << "a*b = " << produit; 
    cout << ", temps coul : " << timer << endl;
  };

  cout << "Division euclidienne : " << flush;
  {
    Naturel quot, reste;
    Timer timer;
    bool ok= division_lente(a,b,quot,reste);
    timer.stop();
    if ( !ok ) cout << "echec";
    else cout << "a/b = " << quot << ", a%b = " << reste; 
    cout << ", temps coul : " << timer << endl;
  };
}

int main()
{
  // Message de bienvenue
  cout << "\nBonjour et bienvenue aux nombres naturels, faits maison !"
       << "\nL'entre/sortie se fait dans la base " << base 
       << " en utilisant \nles chiffres ";
  for( Chiffre i=0; i<base; ++i ) cout << symbole(i) << ",";
  cout << "\b." << endl;

  // Afficher deux exemples
  cout << "\n--> Petit exemple :" << endl;
  test_algorithmes_scolaires( 98765, 12345 );

  cout << "\n--> Exemple plus grand :" << endl;
  Naturel x, y;
  x= "9876543210987654321098765432109876543210";
  y= "1234567890123456789012345678901234567890";
  test_algorithmes_scolaires( x, y );

  // Tester la multiplication par le calcul d'une grande factorielle
  cout << "\n--> Calcul de la factorielle n!" << endl;
  Naturel n(0), factorielle(1);
  for( Naturel min(1), max(100); n<min || n>max ; cin >> n )
    cout << "Entrez n entre " << min << " et " << max << " svp : ";
  for( Naturel k= 1; k<=n; ++k )
    {
      factorielle= factorielle * k;
      cout << k << "! = " << factorielle << endl;
    };

  // Tester l'addition et la multiplication de deux nombres naturels
  cout << "\n--> Comparer les algorithmes scolaires et lents : ";
  for(;;)
    {
      cout << "\nEntrez deux nombres naturels svp (\"0 0\" pour arrter) : ";
      Naturel a, b;
      cin >> a >> b;
      if ( a==0 && b==0 ) break;
      if ( cin.good() ) cout << "status : good" << endl;
      if ( cin.eof() )  cout << "status : eof " << endl;
      if ( cin.fail() ) cout << "status : fail" << endl;
      if ( cin.bad() )  cout << "status : bad " << endl;
      test_algorithmes_scolaires( a, b );
      test_algorithmes_lents( a, b );
    };
  cout << "Au revoir.\n" << endl;
}
