/*=======================================================================*\
 * permut-v02.cc : implmentation minimaliste de permutations
 * Copyright (C) 2003-2005 by Michael Eisermann, all rights reserved
 * Institut Fourier, Universit Joseph Fourier, Grenoble, France
\*=======================================================================*/

#include <iostream>
#include <vector>
using namespace std;

class Permut
{
public:
  typedef short int Point;              // type des points  permuter
  vector<Point> leVecteur;              // pour stocker la permutation
  void raccourcir( void );              // supprimer de points fixes 
  Point degre( void ) const             // dterminer le degr
    { return leVecteur.size(); };

  Point& operator[] ( Point a )         // adresser par rfrence
    { return leVecteur[a]; };

  Point operator[] ( Point a ) const    // valuation (sans contrle)
    { return leVecteur[a]; };

  Point operator() ( Point a ) const    // valuation (avec contrle)
    { return ( a<0 || a>=leVecteur.size() ? a : leVecteur[a] ); };
};

void Permut::raccourcir( void )
{
  Point p= leVecteur.size()-1;
  while ( p >= 0 && leVecteur[p] == p ) --p;
  leVecteur.resize(p+1);
};

Permut operator ! ( const Permut& p )
{
  Permut inverse;
  inverse.leVecteur.resize( p.degre() );
  for( Permut::Point a=p.degre()-1; a>=0; --a ) 
    inverse[ p[a] ]= a;
  return inverse;
};

Permut operator * ( const Permut& p, const Permut& q )
{ 
  // Dterminer le dernier point qui soit boug par le produit p*q
  Permut::Point a= max( p.degre(), q.degre() ) - 1;
  while( a>=0 && p(q(a)) == a ) --a;

  // Calculer le produit p*q (dont le degr vient d'tre dtermin)
  Permut produit;
  produit.leVecteur.resize(a+1);
  for ( ; a>=0; --a )
    produit[a]= p(q(a));
  return produit;
};

ostream& operator << ( ostream& out, const Permut& p )
{ 
  out << "(";
  vector<bool> deja_visite( p.degre(), false );

  // visiter tous les points 
  for ( Permut::Point i=0; i<p.degre(); ++i )
    if ( p[i]!=i && !deja_visite[i] )
      {
	// parcourir chaque cycle non trivial
	out << "(" << i;
	deja_visite[i]= true;
	for( Permut::Point j= p[i]; !deja_visite[j]; j= p[j] )
	  {
	    out << "," << j;
	    deja_visite[j]= true;
	  };
	out << ")";
      };
  out << ")";
  return out;
};

int main()
{
  Permut p;
  p.leVecteur.resize(3);
  p[0]= 1; p[1]= 2; p[2]= 0;
  cout << "p = " << p << endl;
  cout << "!p = " << !p << endl;

  Permut q;
  q.leVecteur.resize(4);
  q[0]= 0; q[1]= 2; q[2]= 3; q[3]= 1;
  cout << "q = " << q << endl;
  cout << "!q = " << !q << endl;
  
  cout << "p*q = " << p*q << endl;
  cout << "q*p = " << q*p << endl;
};
