/*=======================================================================*\
 * permut-v03.cc : implmentation minimaliste de permutations
 * Copyright (C) 2003-2005 by Michael Eisermann, all rights reserved
 * Institut Fourier, Universit Joseph Fourier, Grenoble, France
\*=======================================================================*/

#include <iostream>
#include <vector>
using namespace std;

class Permut
{
private:
  typedef short int Point;              // type des points  permuter
  vector<Point> leVecteur;              // pour stocker la permutation
  void raccourcir( void );              // supprimer de points fixes 
  Point& operator[] ( Point a )         // adresser par rfrence
    { return leVecteur[a]; };

public:
  Permut() {};                          // constructeur : identit par dfaut
  Permut( Point a, Point b )            // constructeur : transposition a<->b) 
    { setTransposition(a,b); };

  Point degre( void ) const             // dterminer le degr
    { return leVecteur.size(); };

  Point operator[] ( Point a ) const    // valuation (sans contrle)
    { return leVecteur[a]; };

  Point operator() ( Point a ) const    // valuation (avec contrle)
    { return ( a<0 || a>=leVecteur.size() ? a : leVecteur[a] ); };

  friend Permut operator ! ( const Permut& p );
  friend Permut operator * ( const Permut& p, const Permut& q );

  void setTransposition( Point a, Point b );
};

void Permut::setTransposition( Point a, Point b )
{ 
  if ( a > b ) swap( a, b );
  if ( a < 0 || a == b ) { leVecteur.clear(); return; };
  leVecteur.resize(b+1);
  for( Point i=0; i<=b; ++i ) leVecteur[i]= i;    
  leVecteur[a]= b; leVecteur[b]= a;
};

void Permut::raccourcir( void )
{
  Point p= leVecteur.size()-1;
  while ( p >= 0 && leVecteur[p] == p ) --p;
  leVecteur.resize(p+1);
};

Permut operator ! ( const Permut& p )
{
  Permut inverse;
  inverse.leVecteur.resize( p.degre() );
  for( Permut::Point a=p.degre()-1; a>=0; --a ) 
    inverse[ p[a] ]= a;
  return inverse;
};

Permut operator * ( const Permut& p, const Permut& q )
{ 
  // Dterminer le dernier point qui soit boug par le produit p*q
  Permut::Point a= max( p.degre(), q.degre() ) - 1;
  while( a>=0 && p(q(a)) == a ) --a;

  // Calculer le produit p*q (dont le degr vient d'tre dtermin)
  Permut produit;
  produit.leVecteur.resize(a+1);
  for ( ; a>=0; --a )
    produit[a]= p(q(a));
  return produit;
};

ostream& operator << ( ostream& out, const Permut& p )
{ 
  out << "(";
  vector<bool> deja_visite( p.degre(), false );

  // visiter tous les points 
  for ( Permut::Point i=0; i<p.degre(); ++i )
    if ( p[i]!=i && !deja_visite[i] )
      {
	// parcourir chaque cycle non trivial
	out << "(" << i;
	deja_visite[i]= true;
	for( Permut::Point j= p[i]; !deja_visite[j]; j= p[j] )
	  {
	    out << "," << j;
	    deja_visite[j]= true;
	  };
	out << ")";
      };
  out << ")";
  return out;
};

int main()
{
  Permut p= Permut(0,1) * Permut(1,2);
  cout << "p = " << p << endl;
  cout << "!p = " << !p << endl;
  Permut q(1,3);
  cout << "q = " << q << endl;
  cout << "p*q = " << p*q << endl;
  cout << "q*p = " << q*p << endl;
};
