//=======================================================================
// euclide.cc : une visualisation de l'algorithme d'Euclide
// Copyright (C) 2004-2007 by Michael Eisermann, all rights reserved
// Institut Fourier, Universit Joseph Fourier, Grenoble, France
//-----------------------------------------------------------------------
// Description :
// Comme la division euclidienne d'entiers n'est pas unique, chaque tape 
// de l'algorithme d'Euclide prsente un choix entre deux alternatives. 
// Ce programme visualise tous les diffrents parcours possibles. 
// Bien entendu le pgcd ainsi calcul est toujours le mme (au signe prs)
// mais la longueur du calcul peut varier considrablement ! 
// Ainsi on peut se convaincre sur des exemples que l'on a tout intrt 
//  choisir systmatiquement une division avec reste minimal.
//=======================================================================

#include <iostream>    // dclarer l'entre-sortie standard
#include <iomanip>     // manipulateurs pour formater les flots
#include <vector>      // pour dfinir la classe gnrique vector
using namespace std;   // accs direct aux fonctions standard

typedef int Integer;

//-----------------------------------------------------------------------
// Dans la fonction euclide_verbeux le vecteur hist contient l'histoire
// de l'algorithme, c'est--dire les calculs effectus prcdemment. 
// On l'appelle initialement avec un vecteur de longueur deux, qui 
// contient les paramtres initiaux a et b. On en calcule le pgcd par 
// des appels rcursifs, en rallongeant chaque fois le vecteur hist.
//  la fin d'un tel calcul on affiche toute l'histoire  l'cran. 
// Les paramtres min et max servent  faire des statistiques :  la fin
// ils contiennent la longueur minimale resp. maximale des calculs.
//-----------------------------------------------------------------------
void euclide_verbeux( vector<Integer>& hist, Integer &min, Integer& max )
{
  // attraper quelques cas errons
  int n= hist.size();
  if ( n < 2 ) return;
  Integer a= hist[n-2];
  Integer b= hist[n-1];
  if ( b == 0 ) return;
  
  // calculer le reste d'une division euclidienne
  Integer r= a%b;

  // quand l'algorithme se termine, afficher le parcours
  if ( r == 0 )
    {
      for ( int i=0; i<hist.size(); ++i )
	cout << setw(5) << hist[i];
      cout << endl;
      if ( hist.size() > max ) max= hist.size();
      if ( hist.size() < min ) min= hist.size();
      return;
    };
  
  // sinon, dterminer les deux restes possibles
  Integer r1, r2;
  if ( r > 0 )
    {
      r1= r;
      r2= r - abs(b);
    }
  else
    {
      r2= r;
      r1= r + abs(b);
    };

  // ordonner les deux restes par leur valeur absolue
  if ( abs(r2) < abs(r1) ) swap(r1,r2);
  
  // suivre les deux branches dans l'algorithme
  hist.push_back( r1 );
  euclide_verbeux( hist, min, max );
  hist.pop_back();
  hist.push_back( r2 );
  euclide_verbeux( hist, min, max );
  hist.pop_back();
}

int main()
{
  cout << "\nBienvenue  l'arbre d'Euclide !" << endl
       << "Comme la division euclidienne d'entiers n'est pas unique," << endl
       << "chaque tape de l'algorithme d'Euclide prsente un choix" << endl
       << "entre deux alternatives. Afin de visualiser un exemple," << endl
       << "entrez deux petits entiers (par exemple 123 99 ou 20 19) : ";
  Integer a,b;
  cin >> a >> b;

  cout.setf( ios::showpos );
  cout << "Calculons donc le pgcd de " << a << " et " << b 
       << " par l'algorithme d'Euclide." << endl;
  if ( a == 0 || b == 0 )
    {
      cout << "Le pgcd est vident." << endl;
      exit(0);
    };

  cout << "Voici les diffrents parcours possibles : " << endl;
  vector<Integer> hist;
  hist.push_back(a);
  hist.push_back(b);
  Integer min= max(abs(a),abs(b)), max= 0;
  euclide_verbeux( hist, min, max );
  cout << "Longueur min=" << min << " max=" << max << endl << endl;
}
