//=======================================================================
// cesar.cc : cryptage alphabtique selon Jules Csar
// Copyright (C) 2007 Michael Eisermann, all rights reserved
// Institut Fourier, Universit Joseph Fourier, Grenoble, France
//=======================================================================

#include <iostream>
#include <string>
using namespace std;

// Transformer un texte en majuscules 'ABC..XYZ'
void majuscule( string& texte )
{
  for( size_t i=0; i<texte.size(); ++i )
    if ( texte[i] >= 'a' && texte[i] <= 'z' ) 
      texte[i]+= 'A' - 'a'; 
}

// Crer une permutation alatoire de l'alphabet ABC...XYZ
string code_aleatoire()
{
  string code("ABCDEFGHIJKLMNOPQRSTUVWXYZ");
  srandom( time(NULL) );
  for( int i=25; i>=1; --i )
    {
      int j= random()%(i+1);
      swap( code[i], code[j] );
    }
  return code;
}

// Dterminer la permutation inverse.  Si la chane n'est pas
// une permutation de l'alphabet, on renvoie une chane vide.
string inverse( const string& code )
{
  string inverse("..........................");
  for( int i=0; i<26; ++i )
    {
      int j= code[i]-'A';
      if( j<0 || j>25 || inverse[j]!='.' ) return string();
      inverse[j]= 'A'+i;
    }
  return inverse;
}

// Crypter un texte par le code donn : on remplace
// chaque lettre suivant la permutation prescrite.
void crypter( string& texte, const string& code )
{
  majuscule( texte );
  for( size_t i=0; i<texte.size(); ++i )
    if( texte[i]>='A' && texte[i]<='Z' )
      texte[i]= code[ texte[i]-'A' ];
}

// Ajouter un bruit alatoire  un message
string bruiter( const string& texte )
{
  string bruite;
  for( size_t i=0; i<texte.size(); ++i )
    if( texte[i]>='A' && texte[i]<='Z' )
      {
	int a= texte[i]-'A';  // la lettre initiale
	int b= random() % 26; // le bruit alatoire
	int c= (a+b) % 26;    // la lettre bruite
	bruite+= ('A'+c);
	bruite+= ('A'+b);
      }
    else
      bruite+= texte[i];
  return bruite;
}

// Soustraire le bruit d'un message
string debruiter( const string& texte )
{
  string debruite;
  for( size_t i=0; i<texte.size(); ++i )
    if( texte[i]>='A' && texte[i]<='Z' )
      {
	int c= texte[i]-'A';   // la lettre bruite
	int b= texte[++i]-'A'; // le bruit alatoire
	int a= (c+26-b) % 26;  // la lettre initiale
	debruite+= ('A'+a);
      }
    else
      debruite+= texte[i];
  return debruite;
}

int main( int nombre_de_parametres, char* liste_des_parametres[] )
{
  // Message de bienvenue / aide en ligne 
  if( nombre_de_parametres == 1 )
    {
      string code= code_aleatoire();
      cout << "\ncesar -- cryptage alphabtique selon Jules Csar\n"
	   << "(C) 50 avJC - 2007 Jules Csar & Michael Eisermann\n"
	   << "usage : cesar [<options>] <texte>\n"
	   << "Le argument <texte> est un texte form de lettres ABC...XYZ\n"
	   << "et ventuellement d'autres caractres neutres.  Options:\n"
	   << "  -b  ajoute un bruit alatoire au texte initial.\n"
	   << "  -d  soustrait le bruit alatoire d'un texte bruit.\n"
	   << "  -c  permet de spcifier le code pour le cryptage/dcryptage.\n"
	   << "Ici le code  utiliser est une permutation de l'alphabet.\n"
	   << "Exemple    : ABCDEFGHIJKLMNOPQRSTUVWXYZ\n"
	   << "cryptage   : " << code << endl
	   << "dcryptage : " << inverse(code) << endl << endl;
      return 0;
    }
  cout << "\nBonjour et bienvenue au cryptage selon Jules Csar !" << endl;

  // Lecture d'ventuelles options
  bool option_bruiter= false, option_debruiter= false;
  string code;
  int n;
  for( n=1; n < nombre_de_parametres; ++n )
    {
      string param( liste_des_parametres[n] );

      if( param == "-b" )
	{
	  cout << "L'option '-b' entrane le bruitage du texte." << endl;
	  option_bruiter= true;
	  continue;
	};

      if( param == "-c" )
	{
	  cout << "L'option '-c' permet de spcifier le code." << endl;	  
	  if( ++n < nombre_de_parametres )
	    {
	      code= liste_des_parametres[n];
	      majuscule( code );
	      cout << "code  : " << code << endl;
	      if ( inverse(code).empty() )
		{
		  cout << "Ce n'est pas une permutation de l'alphabet !\n" << endl;
		  code.clear();
		  --n;
		};
	    }
	  else
	    {
	      cout << "Il manque le code aprs l'option '-c' !" << endl;
	    };
	  continue;
	};
      
      if( param == "-d" )
	{
	  cout << "L'option '-d' entrane le dbruitage du texte." << endl;
	  option_debruiter= true;
	  continue;
	};
      break;
    }

  // Test de plausibilit
  if( option_bruiter && option_debruiter )
    {
      cout << "Les options '-b' et '-d' se contredisent et sont annules." << endl;
      option_bruiter= false;
      option_debruiter= false;
    };
  
  if( code.empty() )
    {
      code= code_aleatoire();
      cout << "Par dfaut j'utiliserai un code choisi alatoirement.\n"
	   << "alphabet   : ABCDEFGHIJKLMNOPQRSTUVWXYZ\n"
	   << "cryptage   : " << code << endl
	   << "dcryptage : " << inverse(code) << endl;
    };
  
  // Lecture du texte (paramtre obligatoire)
  string texte;
  if( n < nombre_de_parametres )
    {
      texte= liste_des_parametres[n++];
      majuscule( texte );
    }
  else
    {
      cout << "Il manque le texte  crypter/dcrypter !" << endl;
      return 1;
    };
  cout << endl << "texte : " << texte << endl;
  
  // Appliquer le bruitage (optionnel)
  if( option_bruiter )
    {
      texte= bruiter( texte );
      cout << "texte bruit : " << texte << endl;
    };

  // Appliquer le cryptage/dcryptage
  crypter( texte, code );
  cout << "texte crypt : " << texte << endl;
  
  // Appliquer le dbruitage (optionnel)
  if( option_debruiter )
    {
      texte= debruiter( texte );
      cout << "texte dbruit : " << texte << endl;
    };

  cout << endl;
}
