template <typename Anneau>
class Quotient
{
private:
  Anneau mod, rep; // le module et le reprsentant 

public:
  // Constructeurs
  Quotient( const Anneau& r=Anneau(0), const Anneau& m=Anneau(0) )
    : mod( assopref(m) ), rep( eumod( r, mod ) ) {};
  Quotient( const Quotient& a )
    : mod(a.mod), rep(a.rep) {};

  // Affectation
  void affecter( const Anneau& r=0, const Anneau& m=0 )
    { mod= assopref(m); rep= eumod( r, mod ); };
  Quotient& operator= ( const Quotient& a )
    { mod= a.mod; rep= a.rep; return *this; };

  // Lire le module et le reprsentant
  const Anneau& module() const { return mod; };
  const Anneau& representant() const { return rep; };

  // Comparaison
  bool operator== ( const Quotient& a ) const
    { return associes( mod, a.mod ) && divisible( rep-a.rep, mod ); }; 
  bool operator!= ( const Quotient& a ) const
    { return !associes( mod, a.mod ) || !divisible( rep-a.rep, mod ); }; 

  // Addition
  Quotient operator+ ( const Quotient& a ) const
    { return Quotient( rep + a.rep, pgcd( mod, a.mod ) ); };
  Quotient& operator+= ( const Quotient& a )
    { affecter( rep + a.rep, pgcd( mod, a.mod ) ); return *this; };

  // Oppos et soustraction
  Quotient operator- () const
    { return Quotient( -rep, mod ); };
  Quotient operator- ( const Quotient& a ) const
    { return Quotient( rep - a.rep, pgcd( mod, a.mod ) ); };
  Quotient& operator-= ( const Quotient& a )
    { affecter( rep - a.rep, pgcd( mod, a.mod ) ); return *this; };

  // Multiplication
  Quotient operator* ( const Quotient& a ) const
    { return Quotient( rep * a.rep, pgcd( mod, a.mod ) ); };
  Quotient& operator*= ( const Quotient& a )
    { affecter( rep * a.rep, pgcd( mod, a.mod ) ); return *this; };
};
