#include <iostream>
#include "integer.cc"
using namespace std;

class Rationnel
{
private:
  Integer numer, denom;   // numrateur et dnominateur
  void normaliser(void)   // normaliser la fraction numer/denom
    { if ( zero(denom) ) { cerr << "Division par zro !\n"; exit(1); };
      Integer d= pgcd(numer,denom); numer/= d; denom/= d;
      if ( denom<0 ) { numer= -numer; denom= -denom; }; };
  
public:
  // Constructeurs divers
  Rationnel( int num=0, int den=1 )
    : numer(num), denom(den) { normaliser(); };
  Rationnel( const Integer& num, const Integer& den=1 )
    : numer(num), denom(den) { normaliser(); };
  Rationnel( const Rationnel& a )
    : numer(a.numer), denom(a.denom) {};

  // Lire le numrateur et le dnominateur
  const Integer& numerateur() const { return numer; };
  const Integer& denominateur() const { return denom; };

  // Affectation
  void affecter( const Integer& num=0, const Integer& den=1 )
    { numer= num; denom= den; normaliser(); };
  Rationnel& operator= ( const Rationnel& a )
    { numer= a.numer; denom= a.denom; return *this; };

  // Addition
  Rationnel operator+ ( const Rationnel& a ) const
    { return Rationnel( numer*a.denom + denom*a.numer, denom*a.denom ); };
  Rationnel& operator+= ( const Rationnel& a )
    { affecter( numer*a.denom + denom*a.numer, denom*a.denom ); return *this; };

  // Oppos et soustraction
  Rationnel operator- () const
    { return Rationnel( -numer, denom ); };
  Rationnel operator- ( const Rationnel& a ) const
    { return Rationnel( numer*a.denom - denom*a.numer, denom*a.denom ); };
  Rationnel& operator-= ( const Rationnel& a )
    { affecter( numer*a.denom - denom*a.numer, denom*a.denom ); return *this; };

  // Multiplication (correcte mais non optimise)
  Rationnel operator* ( const Rationnel& a ) const
    { return Rationnel( numer*a.numer, denom*a.denom ); };
  Rationnel& operator*= ( const Rationnel& a )
    { affecter( numer*a.numer, denom*a.denom ); return *this; };

  // Division (correcte mais non optimise)
  Rationnel operator/ ( const Rationnel& a ) const
    { return Rationnel( numer*a.denom, denom*a.numer ); };
  Rationnel& operator/= ( const Rationnel& a )
    { affecter( numer*a.denom, denom*a.numer ); return *this; };

  // Entre et sortie
  friend istream& operator>> ( istream& in, Rationnel& a );
  friend ostream& operator<< ( ostream& out, const Rationnel& a );
};
