#include <iostream>
#include <vector>
#include "pecoprem.hh"

typedef int Degre;

// Dterminer le degr
template <typename A>
Degre deg( const vector<A>& p )
{ return p.size() - 1; }

// Lire un coefficient
template <typename A>
A coeff( const vector<A>& p, Degre k )
{ if ( k<0 || k>deg(p) ) return A(0); else return p[k]; };

// Normaliser en supprimant d'ventuels zros terminaux
template <typename A>
void normaliser( vector<A>& p )
{ while( !p.empty() && zero(p.back()) ) p.pop_back(); }

// Addition de deux polynmes
template <typename A>
vector<A> operator + ( const vector<A>& p, const vector<A>& q )
{
  Degre d= max( deg(p), deg(q) );
  vector<A> somme( d+1 );
  for ( Degre k=0; k<=d; ++k ) 
    somme[k]= coeff(p,k) + coeff(q,k);
  normaliser(somme);
  return somme;
}

// Multiplications de deux polynmes
template <typename A>
vector<A> operator * ( const vector<A>& p, const vector<A>& q )
{
  if ( deg(p)<0 || deg(q)<0 ) return vector<A>(0);
  vector<A> produit( deg(p) + deg(q) + 1, A(0) );
  for ( Degre i=0; i<=deg(p); ++i )
    for ( Degre j=0; j<=deg(q); ++j )
      produit[i+j]+= p[i]*q[j];
  return produit;
}

// D'autres oprations souhaitables
template <typename A>
vector<A> operator + ( const A& a, const vector<A>& p );

template <typename A>
vector<A> operator + ( const vector<A>& p, const A& a );

template <typename A>
vector<A> operator - ( const vector<A>& p, const vector<A>& q );

template <typename A>
vector<A> operator - ( const A& a, const vector<A>& p );

template <typename A>
vector<A> operator - ( const vector<A>& p, const A& a );

template <typename A>
vector<A> operator * ( const A& a, const vector<A>& p );

template <typename A>
vector<A> operator * ( const vector<A>& p, const A& a );

// Division unitaire
template <typename A>
void div( const vector<A>& a, const vector<A>& b, vector<A>& q, vector<A>& r );

template <typename A>
void mod( const vector<A>& a, const vector<A>& b, vector<A>& r );

template <typename A>
void div( const vector<A>& a, const vector<A>& b, vector<A>& q )
{ vector<A>& r; div(a,b,q,r); };

// Evaluation d'aprs Horner
template <typename A>
A eval( const vector<A>& p, const A& c )
{
  if ( p.empty() ) return A(0);
  A somme= p.back();
  for( Degre k=p.size()-1; k>=0; --k )
    { somme*= c; somme+= p[k]; };
  return somme;
}

typedef GF<11> F;

int main()
{
  vector<F> p,q,r,s;
  eval(p,F(1));
  s= p+q;
  r= p*q;
}
