//=======================================================================
// instable-interval.cc : itration fiable de l'application  x -> 4x(1-x)
// Copyright (C) 2004-2007 Michael Eisermann, all rights reserved
// Institut Fourier, Universit Joseph Fourier, Grenoble, France
//
// Compilation :
// Compiler avec l'option -lgmpxx pour intgrer la bibliothque gmp. 
//
// Description :
// Ce petit programme itre la fonction x -> 4x(1-x).  Compte tenu de 
// l'instabilit numrique, on implmente ici un calcul fiable sous forme
// d'encadrements : on calcule l'image f([x,y]) d'un intervalle [x,y] 
// en arrondissant vers le bas pour x et vers le haut pour y.
//
// Commentaire :
// Vous trouvez le mme calcul dans le fichier instable-interval.cc 
// utilisant directement le type RReal sans intervalles. 
//=======================================================================

#include <iostream>
#include <iomanip>
using namespace std;

#include "interval.cc"
using namespace Numeric;

const RReal milieu= RReal(1)/RReal(2);

// La fonction  f : x -> 4x(1-x)  avec un arrondi fiable
RReal f( const RReal& x, Mode mode= stdMode )
{
  return mul( mul( 4, x, mode), sub( 1, x, mode ), mode );
}

// La fonction f est croissante sur [0,1/2] et dcroissante
// sur [1/2,1].  L'image d'un interval  a = [min,max] s'obtient 
// donc en distinguant les trois cas indiqus suivants.
Interval f( const Interval& a )
{
  if ( a.max() <= milieu )    // premier cas : 0 <= min <= max <= 1/2
    return Interval( f( a.min(), RNDD ), f( a.max(), RNDU ) );
  if ( a.min() >= milieu )    // second cas  : 1/2 <= min <= max <= 1
    return Interval( f( a.max(), RNDD ), f( a.min(), RNDU ) );
  
  // Dans le troisime cas on a   0 <= min < 1/2 < max <= 1
  return Interval( min( f( a.min(), RNDD ), f( a.max(), RNDD ) ), 1 );
}

int main()
{
  cout << "\nBonjour et bienvenue ! Ce programme itre la fonction f(x) = 4x(1-x).\n"
       << "Compte tenu de l'instabilit numrique, il semble honnte de calculer\n"
       << "des encadrements fiables grce aux arrondis dirigs.  Ceci ne rsout\n"
       << "pas l'instabilit, mais garantit au moins la correction de la rponse.\n" << endl;
  Len decimales= demander_la_precision();
  cout.precision( decimales );

  cout << "\nDonner un intervalle a pour la valeur initiale :" << endl;
  Interval a;
  cout << "a = ";
  cin >> a;
  cout << "a = " << a << endl;
  
  const Interval unit_interval(0,1);
  if( !unit_interval.contains(a) )
    {
      cout << "Cet intervalle n'est pas contenu dans [0,1].\n" << endl;
      exit(1);
    }
  
  cout << "\nCombien d'itrations dsirez-vous ? ";
  int iterations;
  cin >> iterations;

  cout << "\nOn effectue deux calculs en parallle -  titre d'avertissement :" << endl
       << "a : on applique soigneusement la fonction f : x -> 4*x*(1-x)  notre intervalle." << endl
       << "b : on applique navement la formule 4*x*(1-x)  notre intervalle." << endl
       << " noter que la premire approche garantit qu'un sous-intervalle de [0,1] est envoy" << endl
       << "sur un sous-intervalle de [0,1], alors que pour l'approche nave ceci est faux." << endl;

  Interval b= a;
  for( int k=1; k<=iterations; ++k )
    {
      cout << "Itration k=" << k << endl;

      // On applique soigneusement la fonction f : x -> 4*x*(1-x)  notre intervalle
      a= f(a);
      cout << "a = " << a << "  cart = " << a.len() << endl;
      
      // On applique naivement la formule 4*x*(1-x)  notre intervalle
      if( !unit_interval.contains(b) )
	cout << "b est dsormais sorti de l'intervalle [0,1]." << endl;
      else
	{
	  b = 4 * b * (1-b);
	  cout << "b = " << b << ", cart = " << b.len() << endl;
	}
    }

  cout << "\nAu revoir et  bientt.\n" << endl;
}
