//=======================================================================
// interval-test.cc :  arithmtique d'intervalles
// Copyright (C) 2007 Michael Eisermann, all rights reserved
// Institut Fourier, Universit Joseph Fourier, Grenoble, France
//
// Compilation :
// Compiler avec l'option -lgmpxx pour intgrer la bibliothque gmp. 
//
// Description :
// La classe Interval illustre comment implmenter, avec des outils 
// trs lmentaires, le calcul fiable sous forme d'intervalles.
// Nous utilisons ici la classe RReal faite maison pour contrler 
// explicitement le mode d'arrondi. 
//=======================================================================

#include "interval.cc"
using namespace Numeric;

//-----------------------------------------------------------------------
// La fonction main() gre la communication avec l'utilisateur
//-----------------------------------------------------------------------
int main()
{
  cout << "\nBienvenue  l'arithmtique des intervalles !" << endl
       << "On effectuera un calcul fiable grce aux arrondis dirigs." << endl;
  Len decimales= demander_la_precision();
  cout.precision( decimales );

  // Oprations lmentaires
  cout << "\nOprations lmentaires... Veuillez entrer deux intervalles svp :" << endl
       << "(Un nombre  virgule flottante comme 0.1 ou 3.1415 sera encadr" << endl
       << "et ainsi interprt comme un intervalle.)" << endl;
  Interval a,b;
  cout << "a = ";
  cin >> a;
  cout << "a = " << a << "   ";
  if( a.isPoint() ) cout << "(Cet intervalle est rduit  un point.)" << endl;
  else cout << "(Cet intervalle est de longueur " << a.len() << ".)" << endl;    

  cout << "b = ";
  cin >> b;
  cout << "b = " << b << "   ";
  if( b.isPoint() ) cout << "(Cet intervalle est rduit  un point.)" << endl;
  else cout << "(Cet intervalle est de longueur " << b.len() << ".)" << endl;    

  cout << "\nExemples de l'arithmtique d'intervalles :" << endl;
  cout << "a+b = " << a+b << endl;
  cout << "a-b = " << a-b << endl;
  cout << "a*b = " << a*b << endl;
  cout << "a/b = " << a/b << endl;

  /*
  cout << "a+1 = " << a+1 << endl;
  cout << "a-1 = " << a-1 << endl;
  cout << "a*2 = " << a*2 << endl;
  cout << "2*a = " << 2*a << endl;
  cout << "a/2 = " << a/2 << endl;
  cout << "2/a = " << 2/a << endl;
  */

  cout << "sqr(a) = " << sqr(a) << endl;
  cout << "a*a    = " << a*a << endl;

  /*
  // Test d'affichage
  cout << "\nOn peut afficher des intervalles comme la valeur mdiane\n"
       << "et la marge d'erreur.  Donnez un nombre rel svp : ";
  RReal x;
  cin >> x;
  RReal eps= 1;
  cout.precision( 16 );
  for( int k=1; k<=50; ++k, eps/=2 )
    {
      Interval a= x + Interval( -eps, +eps );
      Interval::notation( Interval::display_mid_err );
      cout << "eps = " << eps << "  ==>  a = " << a;
      Interval::notation( Interval::display_mid_only );
      cout << "  ou simplement  " << "a = " << a << endl;
    }
  Interval::notation( Interval::display_min_max );
  */

  // Calcul d'une racine
  cout << "\nCalcul de la racine n-ime d'un intervalle." << endl;
  decimales= demander_la_precision();
  cout.precision( decimales );

  cout << "Entrez un intervalle a et un exposant n svp : " << endl;
  cout << "a = ";
  cin >> a;
  cout << "a = " << a << "   ";
  if( a.isPoint() ) cout << "(Cet intervalle est rduit  un point.)" << endl;
  else cout << "(Cet intervalle est de longueur " << a.len() << ".)" << endl;    

  long n;
  cout << "n = ";
  cin >> n;
  cout << "n = " << n << endl;

  cout << "Je calcule la racine r = a^(1/n) :" << endl;
  Interval r= root(a,n);
  cout << "r = " << r << "   ";
  if( r.isPoint() ) cout << "(Cet intervalle est rduit  un point.)" << endl;
  else cout << "(Cet intervalle est de longueur " << r.len() << ".)" << endl;    

  cout << "Pour vrification je calcule s = r^n :" << endl;
  Interval s= power(r,n);
  cout << "s = " << s << "   ";
  if( s.isPoint() ) cout << "(Cet intervalle est rduit  un point.)" << endl;
  else cout << "(Cet intervalle est de longueur " << s.len() << ".)" << endl;    

  if( !s.contains(a) ) 
    { cerr << "Erreur: l'intervalle s ne contient pas l'intervalle initial a !" << endl; exit(1); }

  cout << "\nBilan sur l'accroissement de la marge d'erreur :" << endl;
  cout << "longueur de a = " << a.len() << endl;
  cout << "longueur de s = " << s.len() << endl;
  cout << "diffrence    = " << s.len() - a.len() << endl;

  cout << "\nAu revoir et  bientt.\n" << endl;
}
