//=======================================================================
// somme-rreal.cc : sommation de la srie  terme gnral 1/k
// Copyright (C) 2004-2007 Michael Eisermann, all rights reserved
// Institut Fourier, Universit Joseph Fourier, Grenoble, France
//
// Compilation :
// Compiler avec l'option -lgmpxx pour intgrer la bibliothque gmp. 
//
// Description :
// Ce petit programme calcule la somme de 1/k pour k=1,...,n puis dans
// le sens inverse k=n,...,1. Chaque fois on tablit un encadrement fiable
// en arrondissant convenablement les sommes partielles. Ainsi on obtient
// deux encadrements corrects, mais l'un est plus fin que l'autre.
// Exercice : expliquer la diffrence, puis formuler une conclusion.
//
// Commentaire :
// Vous trouvez le mme calcul dans le fichier somme-interval.cc qui
// profite de l'arithmtique d'intervalles.  C'est strictement la mme
// dmarche et le mme rsultat, seule l'criture est plus commode.
//=======================================================================

#include <iostream>
using namespace std;

#include "rreal.cc"
using namespace Numeric;

int main()
{
  const int max= 10000;
  cout << "\nBonjour et bienvenue ! Je calcule la somme s des termes 1/k^2" << endl
       << "pour k de 1  " << max << " dans le sens croissant puis dcroissant en k." << endl;
  Len decimales= demander_la_precision();
  cout.precision( decimales );

  RReal mino1(0), majo1(0);
  for( int k=1; k<=max; ++k ) mino1.add( div( 1, sqr( RReal(k), RNDU ), RNDD ), RNDD );
  for( int k=1; k<=max; ++k ) majo1.add( div( 1, sqr( RReal(k), RNDD ), RNDU ), RNDU );
  cout << "\nsommation par indices croissants, encadrement fiable :" << endl;
  mino1.write( cout, RNDD );
  cout << " <= s <= ";
  majo1.write( cout, RNDU );
  cout << endl;
  
  RReal mino2(0), majo2(0);
  for( int k=max; k>=1; --k ) mino2.add( div( 1, sqr( RReal(k), RNDU ), RNDD ), RNDD );
  for( int k=max; k>=1; --k ) majo2.add( div( 1, sqr( RReal(k), RNDD ), RNDU ), RNDU );
  cout << "\nsommation par indices dcroissants, encadrement fiable :" << endl;
  mino2.write( cout, RNDD );
  cout << " <= s <= ";
  majo2.write( cout, RNDU );
  cout << endl;

  cout << "\nLes deux encadrements sont corrects," << endl 
       << "mais l'un est plus fin que l'autre !" << endl
       << "Comment expliquer ce phnomne ?\n" << endl;
}
