//=======================================================================
// iter1.cc : application du thorme du point fixe
// Copyright (C) 2005-2007 by Michael Eisermann, all rights reserved
// Institut Fourier, Universit Joseph Fourier, Grenoble, France
// 
// On se propose ici de calculer une valeur approche de l'unique
// solution relle de l'quation  x = cos(x).
// 
// Prrequis mathmatiques :
// Avant de lancer le programme il faut trouver un intervalle [a,b] sur
// lequel s'applique le thorme du point fixe pour notre fonction f :
// sur cet intervalle [a,b] on doit tablir que f est contractante de 
// rapport k < 1, constante qui joue un rle important dans ce logiciel.
// On peut ensuite choisir une valeur initiale x dans [a,b] puis la 
// prcision souhaite (dans les limites du raisonnable, certes, car le 
// type des nombres machine utilis peut imposer de fortes restriction.)
//=======================================================================

#include <iostream>
#include <cmath>
using namespace std;

//-----------------------------------------------------------------------
// Dfinir la fonction  tudier, puis spcifier ses proprits
//-----------------------------------------------------------------------
double f( double x )     // la fonction  tudier
{ return cos(x); }

const double a  = 0.0;   // extrmit gauche de l'intervalle
const double b  = 1.0;   // extrmit droite de l'intervalle
const double k  = 0.85;  // constante de contraction
const double x  = 0.0;   // valeur initiale de l'itration
const double eps= 1e-5;  // prcision souhaite

//-----------------------------------------------------------------------
// Dfinir la fonction  tudier, puis spcifier ses proprits
//-----------------------------------------------------------------------
/*
double f( double x )     // la fonction  tudier
{ return exp(x/3); }

const double a  = 1.8;   // extrmit gauche de l'intervalle
const double b  = 1.9;   // extrmit droite de l'intervalle
const double k  = 0.63;  // constante de contraction
const double x  = 1.85;  // valeur initiale de l'itration
const double eps= 1e-8;  // prcision souhaite
*/

//-----------------------------------------------------------------------
// Dfinir la fonction  tudier, puis spcifier ses proprits
//-----------------------------------------------------------------------
/*
double f( double x )     // la fonction  tudier
{ return 3*log(x); }

const double a  = 4.5;   // extrmit gauche de l'intervalle
const double b  = 4.6;   // extrmit droite de l'intervalle
const double k  = 0.67;  // constante de contraction
const double x  = 4.55;  // valeur initiale de l'itration
const double eps= 1e-8;  // prcision souhaite
*/

//-----------------------------------------------------------------------
// Lancer l'itration avec contrle de la marge d'erreur
//-----------------------------------------------------------------------
int main()
{
  cout.precision(16);
  cout.setf( ios::fixed );

  cout << "\nBonjour et bienvenue au thorme du point fixe !" << endl
       << "On cherche  approcher la solution de cos(x) = x" << endl
       << "par la mthode du point fixe." << endl
       << "Intervalle          : [ " << a << " ; " << b << " ]" << endl
       << "Contraction         : " << k << endl
       << "Valeur initiale     : " << x << endl
       << "Prcision souhaite : " << eps << endl
       << "Itrations: " << endl;
  
  double u= x;         // la valeur courante dans l'itration  
  double v;            // la valeur prcdente
  double diff;         // diffrence entre les deux
  double marge=2*eps;  // la marge d'erreur

  cout.width(4);
  cout.unsetf( ios::showpos );
  cout << 0 << " : ";
  cout.setf( ios::showpos );
  cout << u << endl;
  
  for( int n=1; marge>eps; ++n )
    {
      // L'itration: appliquer la fonction f 
      v= u;
      u= f(u);
      
      // Dterminer la marge d'erreur
      diff= abs( u-v );
      marge= k/(1-k) * diff;
      
      // Afficher les rsultats intermdiaires
      cout.width(4);
      cout.unsetf( ios::showpos );
      cout << n << " : ";
      cout.setf( ios::showpos );
      cout << u;
      cout.unsetf( ios::showpos );
      cout << "  diff=" << diff 
	   << "  marge=" << marge;
      cout.setf( ios::showpos );
      cout << "  encadrement=[ " << u-marge 
	   << " ; " << u+marge << " ]" << endl;
    };
  
  cout << "Au revoir et  bientt.\n" << endl;
}
