//=======================================================================
// newton.cc : application de la mthode de Newton aux polynmes
// Copyright (C) 2005-2007 by Michael Eisermann, all rights reserved
// Institut Fourier, Universit Joseph Fourier, Grenoble, France
// 
// Description :
// Ce petit logiciel lit un polynme et essaie d'en trouver les racines.
// 
// Avertissement : Aucun effort d'optimisation n'est fait ici !
// Il s'agit d'une implmentation exprimentale  but pdagogique.
//=======================================================================

#include <iostream>  // entre-sortie standard via des flots 
#include <fstream>   // entre-sortie standard via des fichiers
#include <string>    // chanes de caractre
using namespace std;

/*
#include "rreal.cc"      // nombres flottants faits maison
using namespace Numeric; // accs  l'espace de noms Numeric
typedef RReal Flo;       // synonyme utilis dans la suite
*/

typedef long double Flo; // nombres flottants standards pour commencer
typedef long int Len;
Len demander_la_precision()
{ cout << "On calcule avec 64 bits soit 20 dcimales environs." << endl; return 20; } 

#include "polynome.cc"   // classe modlisant les polynomes complexes

//-----------------------------------------------------------------------
// Le boolen  verbeux  gre l'affichage des rsultats intermdiaires
//-----------------------------------------------------------------------
bool verbeux= true;

//-----------------------------------------------------------------------
// Calculer la drive d'un polynme
//-----------------------------------------------------------------------
Polynome diff( const Polynome& p0 )
{
  //
  //
  // Exercice : Implmenter la drivation d'un polynme (5 lignes environ)
  //
  //
}

//-----------------------------------------------------------------------
// Mise en facteur : P(X) = Q(X)(X-a) + reste
//-----------------------------------------------------------------------
Polynome fact( const Polynome& p, const C& a )
{
  //
  //
  // Exercice : Implmenter la mise en facteur (5 lignes environ)
  //
  //
}

//-----------------------------------------------------------------------
// Calculer le produit (X-r[0])(X-r[1])...(X-r[n-1])  partir des racines
//-----------------------------------------------------------------------
Polynome prod( const vector<C>& racines )
{
  //
  //
  // Exercice : Implmenter ce produit (5 lignes environ)
  //
  //
}

//-----------------------------------------------------------------------
// Engendrer un nombre alatoire dans l'intervalle [-r,r]
//-----------------------------------------------------------------------
double rand( double radius )
{ return ( rand() * 2 * radius ) / RAND_MAX - radius; }

//-----------------------------------------------------------------------
// Localiser des racines approches d'un polynme
//
// On essaie de localiser une racine approche du polynme p0, puis on 
// la met en facteur. On ritre la procdure pour le polynme quotient q0:
// on cherche d'abord une racine approche de q0, puis on l'affine en 
// rappliquant la mthode de Newton sur le polynme//initial* p0.
// Cette deuxime tape augmente considrablement la prcision !
//
// Pour ces deux tapes il y a donc deux paramtres  choisir: 
// - localisation compte les itrations pour localiser la racine approche 
//   grossire, d'un polynme auxiliaire de degr de plus en plus petit.
// - affinage spcifie le nombre d'itrations sur le polynme initial.
//   Ceci sert  affiner considrablement la qualit de l'approximation.
//-----------------------------------------------------------------------
void factoriser( const Polynome& p0, vector<C>& racines, 
		 int localisation, int affinage )
{
  if ( verbeux ) cout << "Approximation des racines : " << endl;
  racines.clear();                          // effacer le vecteur
  Polynome p1= diff( p0 );                  // polynme driv P'
  Polynome q0= p0;                          // facteur restant Q
  for( int deg= degre(p0); deg>0; --deg )
    {
      Polynome q1= diff( q0 );              // polynme driv Q'
      C x= C( rand(10.0), rand(10.0) );     // point initial alatoire
      
      // Premire localisation (ventuellement grossire) utilisant Q 
      for( int i=0; i<localisation; ++i )   // appliquer Newton  Q
	{ 
	  x= x - q0(x) / q1(x); 
	  if ( verbeux ) cout << x << "   \r" << flush; 
	}
      if ( verbeux ) cout << "premire approximation : " << x << endl;
      
      // Affinage utilisant P (pour ne pas accumuler des erreurs d'arrondi)
      for( int i=0; i<affinage; ++i )       // appliquer Newton  P
	{
	  x= x - p0(x) / p1(x); 
	  if ( verbeux ) cout << x << "   \r" << flush; 
	}
      if ( verbeux ) cout << "               affine : " << x << endl;

      // Mise en facteur de la racine trouve
      racines.push_back( x );               // stocker la racine et
      q0= fact( q0, x );                    // ... la mettre en facteur
    }
}

//-----------------------------------------------------------------------
// Une variante qui prscrit plutt une prcision souhaite
//-----------------------------------------------------------------------
void factoriser( const Polynome& p0, vector<C>& racines, double eps )
{
  //
  // Comme la fonction prcdente mais les boucles "for" sont
  // remplaces par des conditions d'arrt en fonction de eps.
  //
}

//-----------------------------------------------------------------------
// Une application : factorisation d'un polynme donn
//-----------------------------------------------------------------------
void factoriser( const string& filename )
{
  Len decimales= demander_la_precision();
  cout.precision( decimales );

  // Ouvrir le fichier, afficher un message en cas d'erreur 
  ifstream in( filename.c_str() );
  if ( !in ) 
    {
      cout << "Je ne peux pas lire le fichier \"" << filename << "\"." << endl
	   << "Veuillez rendre le polynme accessible  cet endroit." << endl;
      return;
    }
  
  // Lire le polynme du fichier 
  Polynome lePolynome;
  in >> lePolynome;
  in.close();
  
  // Affichage du polynme  tudier 
  int deg= degre( lePolynome );
  cout << "Le polynme  tudier est de degr : " << deg << endl
       << lePolynome << endl;

  // Demander les paramtres pour la localisation / l'affinage
  int localisation, affinage;
  cout << "Veuillez saisir le nombre d'itrations svp" << endl;
  cout << "- pour la premire localisation : ";
  cin >> localisation;
  cout << "- pour l'affinage suivant       : ";
  cin >> affinage;

  // Fixer le format des affichages suivants
  cout.width( decimales+4 ); 
  cout.precision( decimales );
  cout.setf( ios::fixed | ios::showpos );

  // Trouver les racines une par une
  vector<C> lesRacines;
  factoriser( lePolynome, lesRacines, localisation, affinage );
  // factoriser( lePolynome, lesRacines, 1e-12 );

  // Vrification finale
  cout << "Racines trouves : " << endl;
  for( int k=0; k<deg; ++k ) cout << lesRacines[k] << endl;

  cout << "Vrification : distance du polynme produit au polynme initial : " 
       << dist( prod(lesRacines), lePolynome ) << endl;
}  

//-----------------------------------------------------------------------
// Affiner les racines approches initiales par la mthode de Newton.
// C'est utile par exemple si l'on augmente la prcision des flottants
// ou si l'on perturbe le polyme et on veut faire suivre les racines.
//-----------------------------------------------------------------------
void affiner( const Polynome& p0, vector<C>& racines, int affinage )
{
  const Polynome p1= diff( p0 );
  if ( verbeux ) cout << "Affinage des approximations :" << endl;
  for( size_t i=0; i<racines.size(); ++i )
    {
      C x= racines[i];
      for( int k=0; k<affinage; ++k )
	x= x - p0(x) / p1(x);
      if ( verbeux ) cout << racines[i] << " devient " << x << endl;
      racines[i]= x;
    }
}

//-----------------------------------------------------------------------
// Un exemple d'instabilit : racines mal conditionnes
//-----------------------------------------------------------------------
void perturbation()
{
  // Demander les paramtres  l'utilisateur
  cout << "Entrez le degr n svp : ";
  int deg;
  cin >> deg;
  if ( deg < 2 || deg > 1000 ) 
    {
      cout << "J'abandonne." << endl;
      return;
    }

  cout << "Entrez la perturbation eps svp : ";
  double eps;
  cin >> eps;
  cout << "On prend deg=" << deg << " et eps=" << eps << endl;
  
  Len decimales= demander_la_precision();
  cout.precision( decimales );

  // Le polynme initial est construit  partir des racines prescrites
  vector<C> r(deg);
  for( int k=0; k<deg; ++k ) r[k]= Flo(k+1);
  Polynome p= prod( r );
  
  // Un deuxime polynme pour la perturbation  p -> p + eps*q
  Polynome q(deg);
  q[deg-1]= Flo( eps );

  // Fixer le format des affichages suivants
  cout.width( decimales+4 ); 
  cout.precision( decimales );
  cout.unsetf( ios::scientific );
  cout.setf( ios::fixed | ios::showpos );

  Polynome per= p + q;
  affiner( per, r, 50 );
  cout << "Racines trouves pour le polynme perturb : " << endl;
  for( int k=0; k<deg; ++k ) cout << r[k] << endl;

  cout.setf( ios::scientific );
  cout.unsetf( ios::showpos );
  cout << "Vrification : distance du polynme produit au polynme initial : " 
       << dist( prod(r), per ) << endl;
}

//-----------------------------------------------------------------------
// La fonction principale gre la communication avec l'utilisateur
//-----------------------------------------------------------------------
int main()
{
  cout << "\nBonjour et bienvenue aux polynmes." << endl;

  // Un premier test
  cout << "\nPour tester l'implmentation on considre le polynme" << endl
       << "de degr n ayant pour racines les nombres 1,2,...,n" << endl
       << "puis on le perturbe en ajoutant eps*X^(n-1)." << endl;
  perturbation();

  // Une application pratique
  cout << "\nFactorisation du polynme dans \"polynome.txt\"" << endl;
  factoriser( "polynome.txt" );
  
  cout << "Au revoir et  bientt." << endl;
}
