//=======================================================================
// faddeev.cc : inversion de matrice d'aprs Faddeev
// Copyright (C) 2007 by Michael Eisermann, all rights reserved
// Institut Fourier, Universit Joseph Fourier, Grenoble, France
//
// Description : 
// On implmentera en exercice les oprations lmentaires sur des
// matrices, puis la mthode de Faddeev pour inverser une matrice.
//=======================================================================

#include "matrix.cc"
#include "timer.hh"
using namespace std;

//-----------------------------------------------------------------------
// La mthode de Faddeev.
// On calcule le polynme caractristique p[0] + p[1]X + ... + p[n]X^n et,
// le cas chant, l'inverse B d'une matrice A par la mthode de Faddeev.
//-----------------------------------------------------------------------
template<typename T>
void faddeev( const Matrix<T>& a, Matrix<T>& b, vector<T>& p )
{
  //
  // Exercice (20 lignes environ)
  //
}

//-----------------------------------------------------------------------
// Tester la mthode de Faddeev
//-----------------------------------------------------------------------
template <typename T>
void test_faddeev( const string fichier )
{
  // Ouvrir le fichier, afficher un message d'erreur en cas d'chec
  ifstream in( fichier.c_str() );
  if ( !in ) 
    {
      cout << "Je ne peux pas lire le fichier \"" << fichier << "\"." << endl
	   << "Veuillez rendre la matrice accessible  cet endroit." << endl;
      return;
    }
  
  // Lire la matrice du fichier
  Matrix<T> a;
  in >> a;
  in.close();

  cout << "J'ai lu une matrice de " 
       << a.rowdim() << " lignes et "
       << a.coldim() << " colonnes : " 
       << endl << "A = " << endl << a << endl;
  
  // Calculer l'inverse de a par la mthode de Faddeev (exercice)
  if ( a.rowdim() == a.coldim() )
    {
      cout << "On applique la mthode de Faddeev..." << endl;
      Matrix<T> b;
      vector<T> poly;
      Timer timer;
      timer.start();
      faddeev(a,b,poly);
      timer.stop();
      cout << "temps coul : " << timer << endl;

      // Afficher le polynme caractristique p
      cout << "Polynme caractristique : p = ( ";
      int n= poly.size();
      for( int i=0; i<n; ++i ) cout << poly[i] << " ";
      cout << ")" << endl;
      
      // Calculer p(A), ce qui devrait tre la matrice nulle
      Matrix<T> id( a.rowdim(), T(1) ), p( poly[n-1]*id );
      for( int i=n-2; i>=0; --i ) p= p*a + poly[i]*id;
      cout << "Vrification du thorme de Cayley : p(A) = " << endl << p << endl;
      
      // Afficher l'inverse b de a puis vrifier a*b et b*a
      cout << "L'inverse de a vaut b = " << endl << b << endl;
      if ( b.empty() ) return;
      cout << "Vrification : a*b = " << endl << a*b << endl
	   << "Vrification : b*a = " << endl << b*a << endl;
    }
}

//-----------------------------------------------------------------------
// Un test numrique : matrices de Vandermonde
//-----------------------------------------------------------------------
template <typename T>
void test_vandermonde( void )
{
  char c;
  Matrix<T> a,b;
  vector<T> poly;
  for ( Dim n=6; n<=8; ++n )
    {
      cout << "\nMatrice de Vandermonde de dimension n = " << n << endl;
      a= vandermonde<T>(n);
      cout << "La matrice a vaut : a =" << endl << a << endl;

      cout << "On applique la mthode de Faddeev..." << endl;
      Timer timer;
      timer.start();
      faddeev(a,b,poly);
      timer.stop();
      cout << "temps coul : " << timer << endl;

      cout << "L'inverse de a vaut : b =" << endl << b << endl;
      cout << "Vrification : a*b =" << endl << a*b << endl;
      cout << "Vrification : b*a =" << endl << b*a << endl;

      cout << "Tapez ENTREE pour continuer svp : "; cin.get(c);
      system("clear");
    }
}

//-----------------------------------------------------------------------
// Un autre test numrique : matrices de Hilbert
//-----------------------------------------------------------------------
template <typename T>
void test_hilbert( void )
{
  char c;
  Matrix<T> a,b;
  vector<T> poly;
  for ( Dim n=3; n<=6; ++n )
    {
      cout << "\nMatrice de Hilbert de dimension n = " << n << endl;
      a= hilbert<T>(n);
      cout << "La matrice a vaut : a =" << endl << a << endl;

      cout << "On applique la mthode de Faddeev..." << endl;
      Timer timer;
      timer.start();
      faddeev(a,b,poly);
      timer.stop();
      cout << "temps coul : " << timer << endl;

      cout << "L'inverse de a vaut : b =" << endl << b << endl;
      cout << "Vrification : a*b =" << endl << a*b << endl;
      cout << "Vrification : b*a =" << endl << b*a << endl;

      cout << "Tapez ENTREE pour continuer svp : "; cin.get(c);
      system("clear");
    }
}

//-----------------------------------------------------------------------
// Le choix des scalaires : 
// Pour le calcul numrique on choisira en gnral le type double.
// Pour comparer avec un calcul exact on utilise ici le type Rationnel.
//  compiler avec -lgmpxx car on utilise la bibliothque GMP.
//-----------------------------------------------------------------------
#include <cmath>
typedef double Flottant;

#include <gmpxx.h>
typedef mpq_class Rationnel;

#include "rreal.cc"
using namespace Numeric;

//-----------------------------------------------------------------------
// Un exemple d'utilisation et un premier test.
//-----------------------------------------------------------------------
int main()
{
  char c; 
  system("clear");
  cout << "\nBonjour et bienvenue aux matrices et la mthode de Faddeev !" << endl;

  // Exemple d'une matrice quelconque donne dans un fichier
  cout << "\n*** Mthode de Faddeev utilisant des rationnels ***" << endl;
  test_faddeev<Rationnel>( "exemple.mat" );
  cout << "Tapez ENTREE pour continuer svp : "; cin.get(c);
  system("clear");

  cout << "\n*** Mthode de Faddeev utilisant des flottants  53bits ***" << endl;
  test_faddeev<Flottant>( "exemple.mat" );
  cout << "Tapez ENTREE pour continuer svp : "; cin.get(c);
  system("clear");

  cout << "\n*** Mthode de Faddeev utilisant la classe RReal ***" << endl;
  demander_la_precision(); cin.get(c);
  test_faddeev<RReal>( "exemple.mat" );
  cout << "Tapez ENTREE pour continuer svp : "; cin.get(c);
  system("clear");

  // Exemple de matrices de Vandermonde
  cout << "\n*** Mthode de Faddeev utilisant des rationnels ***" << endl;
  test_vandermonde<Rationnel>();
  cout << "\n*** Mthode de Faddeev utilisant des flottants  53bits ***" << endl;
  test_vandermonde<Flottant>();
  cout << "\n*** Mthode de Faddeev utilisant la classe RReal ***" << endl;
  demander_la_precision();
  test_vandermonde<RReal>();

  // Exemple de matrices de Hilbert
  cout << "\n*** Mthode de Faddeev utilisant des rationnels ***" << endl;
  test_hilbert<Rationnel>();
  cout << "\n*** Mthode de Faddeev utilisant des flottants  53bits ***" << endl;
  test_hilbert<Flottant>();
  cout << "\n*** Mthode de Faddeev utilisant la classe RReal ***" << endl;
  demander_la_precision();
  test_hilbert<RReal>();

  cout << "Au revoir et  bientt." << endl;
}
 
